@@ -15,9 +15,11 @@
 
 """Storage backend for S3 or Storage Servers that follow the S3 Protocol"""
 
+import io
 import logging
 import math
 import re
+import urllib
 
 from boto3 import session as boto_session
 from botocore import client as boto_client
@@ -27,8 +29,6 @@ import eventlet
 from oslo_config import cfg
 from oslo_utils import encodeutils
 from oslo_utils import units
-import six
-from six.moves import urllib
 
 import glance_store
 from glance_store import capabilities
@@ -64,6 +64,22 @@ Related Options:
     * s3_store_access_key
     * s3_store_secret_key
 
+"""),
+    cfg.StrOpt('s3_store_region_name',
+               default='',
+               help="""
+The S3 region name.
+
+This parameter will set the region_name used by boto.
+If this parameter is not set, we we will try to compute it from the
+s3_store_host.
+
+Possible values:
+    * A valid region name
+
+Related Options:
+    * s3_store_host
+
 """),
     cfg.StrOpt('s3_store_access_key',
                secret=True,
@@ -375,6 +391,7 @@ class Store(glance_store.driver.Store):
         itself, it should raise `exceptions.BadStoreConfiguration`
         """
         self.s3_host = self._option_get('s3_store_host')
+        self.region_name = self._option_get('s3_store_region_name')
         self.access_key = self._option_get('s3_store_access_key')
         self.secret_key = self._option_get('s3_store_secret_key')
         self.bucket = self._option_get('s3_store_bucket')
@@ -432,6 +449,8 @@ class Store(glance_store.driver.Store):
         if not result:
             if param == 's3_store_create_bucket_on_put':
                 return result
+            if param == 's3_store_region_name':
+                return result
             reason = _("Could not find %s in configuration options.") % param
             LOG.error(reason)
             raise exceptions.BadStoreConfiguration(store_name="s3",
@@ -460,7 +479,10 @@ class Store(glance_store.driver.Store):
             raise boto_exceptions.InvalidDNSNameError(bucket_name=bucket_name)
 
         region_name, endpoint_url = None, None
-        if location:
+        if self.region_name:
+            region_name = self.region_name
+            endpoint_url = s3_host
+        elif location:
             region_name = location
         else:
             endpoint_url = s3_host
@@ -578,7 +600,8 @@ class Store(glance_store.driver.Store):
             if self._option_get('s3_store_create_bucket_on_put'):
                 self._create_bucket(s3_client,
                                     self._option_get('s3_store_host'),
-                                    bucket)
+                                    bucket,
+                                    self._option_get('s3_store_region_name'))
             else:
                 msg = (_("The bucket %s does not exist in "
                          "S3. Please set the "
@@ -706,7 +729,7 @@ class Store(glance_store.driver.Store):
                     checksum.update(write_chunk)
                     if verifier:
                         verifier.update(write_chunk)
-                    fp = six.BytesIO(write_chunk)
+                    fp = io.BytesIO(write_chunk)
                     fp.seek(0)
                     part = UploadPart(mpu, fp, cstart + 1, len(write_chunk))
                     pool.spawn_n(run_upload, s3_client, bucket, key, part)
@@ -721,7 +744,7 @@ class Store(glance_store.driver.Store):
                     checksum.update(write_chunk)
                     if verifier:
                         verifier.update(write_chunk)
-                    fp = six.BytesIO(write_chunk)
+                    fp = io.BytesIO(write_chunk)
                     fp.seek(0)
                     part = UploadPart(mpu, fp, cstart + 1, len(write_chunk))
                     pool.spawn_n(run_upload, s3_client, bucket, key, part)
@@ -850,15 +873,20 @@ class Store(glance_store.driver.Store):
             return True
 
     @staticmethod
-    def _create_bucket(s3_client, s3_host, bucket):
+    def _create_bucket(s3_client, s3_host, bucket, region_name=None):
         """Create bucket into the S3.
 
         :param s3_client: An object with credentials to connect to S3
         :param s3_host: S3 endpoint url
         :param bucket: S3 bucket name
+        :param region_name: An optional region_name. If not provided, will try
+               to compute it from s3_host
         :raises: BadStoreConfiguration if cannot connect to S3 successfully
         """
-        region = get_s3_location(s3_host)
+        if region_name:
+            region = region_name
+        else:
+            region = get_s3_location(s3_host)
         try:
             s3_client.create_bucket(
                 Bucket=bucket,
@@ -888,7 +916,7 @@ class Store(glance_store.driver.Store):
                 {
                     'PartNumber': pnum,
                     'ETag': etag
-                } for pnum, etag in six.iteritems(pedict)
+                } for pnum, etag in pedict.items()
             ]
         }
 
@@ -901,6 +929,7 @@ def get_s3_location(s3_host):
               Amazon S3, and if user wants to use S3 compatible storage,
               returns ''
     """
+    # NOTE(arnaud): maybe get rid of hardcoded amazon stuff here?
     locations = {
         's3.amazonaws.com': '',
         's3-us-east-1.amazonaws.com': 'us-east-1',
