@@ -16,6 +16,7 @@
 """Tests the S3 backend store"""
 
 import hashlib
+import io
 from unittest import mock
 import uuid
 
@@ -25,7 +26,6 @@ from botocore import exceptions as boto_exceptions
 from botocore import stub
 from oslo_utils.secretutils import md5
 from oslo_utils import units
-import six
 
 from glance_store._drivers import s3
 from glance_store import capabilities
@@ -41,6 +41,7 @@ FIVE_KB = 5 * units.Ki
 S3_CONF = {
     's3_store_access_key': 'user',
     's3_store_secret_key': 'key',
+    's3_store_region_name': '',
     's3_store_host': 'localhost',
     's3_store_bucket': 'glance',
     's3_store_large_object_size': 9,        # over 9MB is large
@@ -84,12 +85,35 @@ class TestStore(base.StoreBaseTest,
             self.assertRaises(boto_exceptions.InvalidDNSNameError,
                               self.store.get, loc)
 
+    @mock.patch('glance_store.location.Location')
+    @mock.patch.object(boto3.session.Session, "client")
+    def test_client_custom_region_name(self, mock_client, mock_loc):
+        """Test a custom s3_store_region_name in config"""
+        self.config(s3_store_host='http://example.com')
+        self.config(s3_store_region_name='regionOne')
+        self.config(s3_store_bucket_url_format='path')
+        self.store.configure()
+
+        mock_loc.accesskey = 'abcd'
+        mock_loc.secretkey = 'efgh'
+        mock_loc.bucket = 'bucket1'
+
+        self.store._create_s3_client(mock_loc)
+
+        mock_client.assert_called_with(
+            config=mock.ANY,
+            endpoint_url='http://example.com',
+            region_name='regionOne',
+            service_name='s3',
+            use_ssl=False,
+        )
+
     @mock.patch.object(boto3.session.Session, "client")
     def test_get(self, mock_client):
         """Test a "normal" retrieval of an image in chunks."""
         bucket, key = 'glance', FAKE_UUID
         fixture_object = {
-            'Body': six.BytesIO(b"*" * FIVE_KB),
+            'Body': io.BytesIO(b"*" * FIVE_KB),
             'ContentLength': FIVE_KB
         }
         fake_s3_client = botocore.session.get_session().create_client('s3')
@@ -169,7 +193,7 @@ class TestStore(base.StoreBaseTest,
             S3_CONF['s3_store_host'],
             S3_CONF['s3_store_bucket'],
             expected_image_id)
-        image_s3 = six.BytesIO(expected_s3_contents)
+        image_s3 = io.BytesIO(expected_s3_contents)
 
         fake_s3_client = botocore.session.get_session().create_client('s3')
 
@@ -221,7 +245,7 @@ class TestStore(base.StoreBaseTest,
             S3_CONF['s3_store_host'],
             S3_CONF['s3_store_bucket'],
             expected_image_id)
-        image_s3 = six.BytesIO(expected_s3_contents)
+        image_s3 = io.BytesIO(expected_s3_contents)
 
         fake_s3_client = botocore.session.get_session().create_client('s3')
 
@@ -262,7 +286,7 @@ class TestStore(base.StoreBaseTest,
         expected_image_id = str(uuid.uuid4())
         expected_s3_size = FIVE_KB
         expected_s3_contents = b"*" * expected_s3_size
-        image_s3 = six.BytesIO(expected_s3_contents)
+        image_s3 = io.BytesIO(expected_s3_contents)
 
         fake_s3_client = botocore.session.get_session().create_client('s3')
         verifier = mock.MagicMock(name='mock_verifier')
@@ -294,7 +318,7 @@ class TestStore(base.StoreBaseTest,
             S3_CONF['s3_store_host'],
             S3_CONF['s3_store_bucket'],
             expected_image_id)
-        image_s3 = six.BytesIO(expected_s3_contents)
+        image_s3 = io.BytesIO(expected_s3_contents)
 
         fake_s3_client = botocore.session.get_session().create_client('s3')
 
@@ -375,7 +399,7 @@ class TestStore(base.StoreBaseTest,
         """Tests that adding an image with an existing identifier
         raises an appropriate exception
         """
-        image_s3 = six.BytesIO(b"never_gonna_make_it")
+        image_s3 = io.BytesIO(b"never_gonna_make_it")
 
         fake_s3_client = botocore.session.get_session().create_client('s3')
 
